package petruna.sps;

import java.applet.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.util.List;
import java.util.ArrayList;
import java.io.*;
import java.net.*;

import netscape.javascript.*;
import java.awt.*;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.util.EntityUtils;




public class FileUploadApplet extends Applet implements DropTargetListener, ActionListener
{

	private static final long serialVersionUID = 123282843598L;

	/**
	 * Show files selected to be uploaded so far, info about them, etc.
	 * File could be dragged and dropped on this component, 
	 * what will cause adding file to the list of files for upload.
     */
    private JTextPane dropDisplayPane;

    /**
     * The cached content of the EditorPane
     */
    private StringBuffer dropDisplayPaneContent = new StringBuffer("");    
    
    /**
     * The scroll area in which the dropLabel is embedded
     */
    private JScrollPane scrollPane;    
    
    /**
     * The drop target
     */
    private DropTarget dropTarget;    
    
    /**
     * This is the button which starts the upload process
     */
    private JButton uploadButton;

    /**
     * This is the button which cancel the upload process
     */
    private JButton cancelButton;    
   
    /**
     * This is the button which open a standard file-chooser dialog window
     */
    private JButton fileSelectButton;    
    
    /**
     * The progress bar show the progress of current upload
     */
    private JProgressBar progressBar;
    
    /**
     * This is the list containing files to be uploaded
     */
    private ArrayList fileList = new ArrayList();

    /**
     * the window object in which the applet is embedded
     */
    private JSObject window = null;
    
    /**
     * the thread that uploads the files
     */
    private Thread uploaderThread;
     
   
    /**
     * The init method 
     */
    public void init()
    {
   	
      try
      {
        window = JSObject.getWindow(this);
      }
      catch ( JSException e1 )
      {
        System.out.println("Cannot get a reference to the container window:" + e1.getMessage() );
      }
    
      
      /*
       * set the layout for the applet
       */
      setLayout(new BorderLayout());

      /* 
       * the drop area displays the files
       * for upload and additional information before/during/after update
       */
      dropDisplayPane  = new JTextPane();
      dropDisplayPane.setContentType("text/html");
      dropDisplayPane.setEditable(false);
      scrollPane = new JScrollPane(dropDisplayPane);
      scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
      dropTarget = new DropTarget(dropDisplayPane, this);
      add(scrollPane, BorderLayout.CENTER);        
      
      uploadButton = new JButton("Upload");
      uploadButton.addActionListener(this);
      uploadButton.setEnabled(false);
      
      cancelButton = new JButton("Clear");
      cancelButton.addActionListener(this);
      cancelButton.setEnabled(false);
      
      progressBar = new JProgressBar(0,100);
      progressBar.setEnabled(false);
      progressBar.setStringPainted(true);
      
      fileSelectButton = new JButton("Add file");
      fileSelectButton.addActionListener(this);
      fileSelectButton.setEnabled(true);
      
      /*
       * Button panel
       */
      JPanel panel = new JPanel();
      
      panel.setLayout(new GridBagLayout());
      GridBagConstraints c = new GridBagConstraints();
      c.fill = GridBagConstraints.HORIZONTAL;
      c.weightx = 0.0;
      c.gridwidth = 3;
      c.gridx = 0;
      c.gridy = 0;
      panel.add(fileSelectButton,c);
      c.gridy = 1;
      panel.add(uploadButton,c);
      c.gridy = 2;
      panel.add(cancelButton,c);
      c.gridy = 3;
      panel.add(progressBar,c);
      add(panel, BorderLayout.SOUTH);
      
      setSize(400, 300);
      
      display("<br><br><center> You can drag files you want to upload here <br><br> or you can use add file button below");    
    }

 
    
    /**
     * the action that is called when a button is pressed 
     */
    public void actionPerformed(ActionEvent e)
    {  
    
    	/*
    	 * uploadButton pressed
    	 */
       if ( e.getSource() == uploadButton )
       {
         uploadFiles();
       }
   	    /*
   	     * fileSelectButton pressed
   	     */
       else if ( e.getSource() == fileSelectButton )
       {
    	   JFileChooser jfc = new JFileChooser();
   		   int action = jfc.showDialog(null, "Add file for upload");
   		   if (action == JFileChooser.APPROVE_OPTION) {
   		   File file = jfc.getSelectedFile();
   	       if ( ! fileList.contains(file) ) {
   	       		fileList.add(file);
   	       	  }  	       
           RefreshAfterFileAdd();
   		   }
       }
   		/*
   		 * cancelButton pressed
   		 */
       else if ( e.getSource() == cancelButton )
       {
    	 /*
    	  * if there is active upload to be cancelled
    	  */
         if ( uploaderThread != null && uploaderThread.isAlive() )
         {
           SwingUtilities.invokeLater( new Runnable(){
             public void run()
             {
               cancelButton.setText("Cancelling upload..");
               cancelButton.setEnabled(false);
               display("Cancelling upload...");
             }
           });           
           uploaderThread.interrupt();
          try
          {
            uploaderThread.join();         
          }
          catch (InterruptedException e1){}
          finally
          {
            display("<p><font color=red>Upload interrupted.</font></p>"); 
            resetInput();
          }
         }
     	/*
     	 * none of above pressed
     	 */
         else
         {
           display(null);
           resetInput();
         }
       }
    }
    
    /**
     * write message to dropLabel and scroll to bottom
     */
    /*private*/public void display( String msg )
    {
      if ( msg == null )
      {
        dropDisplayPaneContent = new StringBuffer("");
      }
      else
      {
        dropDisplayPaneContent.append(msg);
      }
      /*
       * behaviour like html page
       */
      String html = "" 
        + "<html>"
        + "<head><style>body { font: Arial 8px }</style></head>"
        + "<body>"
        + dropDisplayPaneContent.toString()
        + "</body>"
        + "</html>";
      dropDisplayPane.setText( html );
    }
    
    /**
     * resets the user interface
     */
    public void resetInput()
    {
      fileList.clear();
      uploadButton.setText("Upload");
      uploadButton.setEnabled(false);
      cancelButton.setText("Clear");
      cancelButton.setEnabled(false);
    }
    
    /**
     * This method handles uploading the selected files to the server.
     */
    private void uploadFiles()
    {
    	
    final	FileUploadApplet fileUploadApplet =this;
      /*
       * we start a new thread for the loop
       */
      uploaderThread = new Thread( new Runnable()
      {
        public void run()
        {
          /*
           * clear display
           */
          display(null);
                   
          /*
           * update applet display
           */
          uploadButton.setText("Uploading files, please wait...");
          cancelButton.setText("Cancel upload");
          uploadButton.setEnabled(false);
          
           
    
           
           UploadDisplayer thisAppletDisplayer = new AppletUploadDisplayer(fileUploadApplet,progressBar);
                 
           FileUploader myMultipartEntityFileUploader =new MyMultipartEntityFileUploader(thisAppletDisplayer,"default");             
             
           myMultipartEntityFileUploader.UploadFiles(fileList);
          

           
          /*
           * reset list and display
           */
          fileList.clear();
          uploadButton.setText("Upload finished.");
          cancelButton.setText("Clear");
          cancelButton.setEnabled(false);         
        }

      } );
                
      /*
       * finally start thread with run method defined above
       */
      uploaderThread.start();
    }

    
    public void dragExit(DropTargetEvent dte)
    {
        // nothing
    }

    public void dragEnter(DropTargetDragEvent dtde)
    {
    	// nothing
    }

    public void dragOver(DropTargetDragEvent dtde)
    {
    	// nothing
    }

    public void dropActionChanged(DropTargetDragEvent dtde)
    {
    	// nothing
    }

    /**
     * This method will be called when the user drops a file on our target label.
     * 
     * @param dtde
     */
    public void drop(DropTargetDropEvent dtde)
    {
        int action = dtde.getDropAction();

        /*
         * We have to tell Java that we are going to accept this drop before
         * we try to access any of the data.
         */
        dtde.acceptDrop(action);

        fromTransferable(dtde.getTransferable());

        /*
         * Once the drop event is complete we need to notify Java again so it
         * can reset the cursor and finish showing the drop behavior.
         */
        dtde.dropComplete(true);
    }

    /**
     * This is a helper method to get the data from the drop event.
     * 
     * @param t
     */
    private void fromTransferable(Transferable t)
    {
        if (t == null) return;

        /*
         * The user may have dropped a file or anything else from any application
         * running on their computer.  This interaction is handled with data flavors.
         */
        try 
        {

          if ( t.isDataFlavorSupported( DataFlavor.javaFileListFlavor) ) 
          {

            List list = (List) t.getTransferData(DataFlavor.javaFileListFlavor);
            
            /*
             * only add to the list if not already in the list.
             */
            if ( ! fileList.containsAll(list) )
            {
              fileList.addAll(list);
              uploadButton.setText("Upload " + fileList.size() + " files.");
            }         
           RefreshAfterFileAdd();         
          } 
          else 
          {
            /*
             * something wrong, ignore
             */
          }
        } 
        catch (NullPointerException npe)
        {
          window.call("alert", new Object[] { "Internal error. Please restart your browser and make sure the upload applet is only running in  browser window."  } );
        }
        catch (Exception ex) 
        {
          ex.printStackTrace();
        }
    }


    /**
     * This method is called after som new files have been added for upload
     */
	private void RefreshAfterFileAdd() {
	       /*
	        * update upload button text
	        */
	       uploadButton.setText("Upload " + fileList.size() + " files.");
         	       

	    /*
	     * clear display
	     */
        display(null); 
        
        for (int i = 0; i < fileList.size(); i++) 
        {
          File f = (File) fileList.get(i);
          String fileName = f.getName();
          if(f.length()>1024){
          int fileSize = (int) Math.floor(  (double) f.length() / 1024 );
          display( fileName + " (" + fileSize + " kB)<br>" );
          }else{
              display( fileName + " (" + f.length() + " B)<br>" );  
          }
        }
	       
        uploadButton.setEnabled(true);
        cancelButton.setEnabled(true);      
		
	}
}
